import 'package:stateful_provider/provider/task_provider.dart';
import 'package:uuid/uuid.dart';

import 'process_status_provider.dart';

class BaseProvider extends TaskProvider {
  final ProcessStatusProvider _status = ProcessStatusProvider();

  Future<T?> enqueue<T>(Future<T> Function() task,
      {bool ignoreIfLoading = false}) async {
    if (ignoreIfLoading && isProcessing) return null;

    final job = Uuid();
    processes.add(job);
    _status.add(job);

    final result = await task().catchError((error) {
      _status.catchError(error);
      return null;
    }).whenComplete(() {
      processes.remove(job);
      _status.complete(job);
    });

    return result;
  }

  void catchError(dynamic source) {
    _status.catchError(source);
  }
}
