import 'package:uuid/uuid.dart';
import 'task_provider.dart';

class ProcessStatusProvider extends TaskProvider {
  factory ProcessStatusProvider() => _singleton;

  static final ProcessStatusProvider _singleton =
      ProcessStatusProvider._internal();

  ProcessStatusProvider._internal();

  Map<Uuid, dynamic> _exceptions = {};

  void add(Uuid uuid) {
    processes.add(uuid);
    _rebuildIfNeeded();
  }

  void complete(Uuid uuid) {
    processes.remove(uuid);
    _rebuildIfNeeded();
  }

  bool _isProcessing = false;

  void _rebuildIfNeeded() {
    final current = isProcessing;
    if (current == _isProcessing) return;
    _isProcessing = current;
    notifyListeners();
  }

  bool _handlingError = false;

  bool get hasError => _exceptions.isNotEmpty;

  int get totalErrors => _exceptions.values.length;

  dynamic currentError() {
    _handlingError = true;
    return _exceptions.values.first;
  }

  void catchError(dynamic error) {
    _exceptions[error.uuid] = error;
    if (_handlingError) return;
    notifyListeners();
  }

  void consumeError(dynamic error) {
    _exceptions.remove(error.uuid);
    _handlingError = false;
    notifyListeners();
  }
}
