import 'package:flutter/material.dart';
import 'package:provider/provider.dart';
import 'package:provider/single_child_widget.dart';
import 'package:stateful_provider/provider/process_status_provider.dart';
import 'package:stateful_provider/widget/process_status_widget.dart';
import 'package:uuid/uuid.dart';

final uuid = Uuid();
int count = 0;

void main() {
  List<SingleChildWidget> providers = [
    ChangeNotifierProvider(create: (_) => ProcessStatusProvider())
  ];

  runApp(
    MaterialApp(
      theme: ThemeData.dark(),
      home: MultiProvider(
        providers: providers,
        child: Scaffold(
          appBar: AppBar(
            title: Text("Stateful provider"),
          ),
          body: Consumer<ProcessStatusProvider>(
            builder: (context, provider, __) => Stack(
              children: [
                Action(provider),
                ProcessStatusWidget(
                  error: (error, _, close) {
                    Scaffold.of(context)
                        .showSnackBar(SnackBar(
                          content: Text("$error"),
                          duration: Duration(seconds: 1),
                        ))
                        .closed
                        .then((value) => close());
                  },
                  process: LinearProgressIndicator(),
                )
              ],
            ),
          ),
        ),
      ),
    ),
  );
}

Widget Action(ProcessStatusProvider status) {
  return Center(
    child: Column(
      mainAxisAlignment: MainAxisAlignment.center,
      children: [
        RaisedButton(
          child: Text("Add process"),
          onPressed: () => status.add(uuid),
        ),
        RaisedButton(
          child: Text("Remove process"),
          onPressed: () => status.complete(uuid),
        ),
        RaisedButton(
          child: Text("Add error"),
          onPressed: () => status.catchError(Exception(["Error# ${count++}"])),
        ),
      ],
    ),
  );
}
